/**
 * Copyright (C) 2007-2012 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <libxfce4util/libxfce4util.h>

#include "services.h"

static gint
_comp (gconstpointer a, gconstpointer b)
{
    return strcmp ((const gchar *)a, (const gchar *)b);
}

static void
_gen_str (gpointer data, gpointer userdata)
{
    gchar *item = (gchar *) data;
    gchar **str = (gchar **) userdata;
    gchar *new_str = NULL;

    if (*str)
        new_str = g_strconcat (*str, " ", item, NULL);
    else
        new_str = g_strdup (item);

    g_free (*str);
    *str = new_str;
}

static void
_list_free (gpointer a, gpointer b)
{
    g_free (a);
}

gboolean
_services_blacklist_write (GList *list)
{
    gchar  *str = NULL;
    gchar  *data = NULL;
    gboolean ret = TRUE;
    XfceRc *rc;

    g_list_foreach (list, _gen_str, &str);
    if (str) {
        data = g_shell_quote (str);
        g_free (str);
    } else {
        data = g_strdup("");
    }

    rc = xfce_rc_simple_open (SERVICES_CONFFILE, FALSE);
    if (! rc) {
        ret = FALSE;
        goto end;
    }
    xfce_rc_write_entry (rc, "SERVICES_BLACKLIST", data);
    xfce_rc_close (rc);

end:
    g_free (data);
    return ret;
}

GList *
services_is_blacklisted (GList *list, const gchar *service)
{
    GList *l = NULL;
    l = g_list_find_custom (list, (gconstpointer) service, _comp);

    return l;
}

static void
_service_run (const gchar *service, gboolean run)
{
    int status;
    gchar *cmd = NULL;

    cmd = g_strdup_printf ("service %s %s",
                           service,
                           run ? "start" : "stop");
    g_spawn_command_line_sync (cmd, NULL, NULL, &status, NULL);
    g_free (cmd);
}

/*
 * Public API
 */
GList *
services_blacklist_list ()
{
    GList  *list = NULL;
    gchar  *str = NULL;
    gchar **a = NULL;
    gchar **tmp;
    XfceRc *rc;

    rc = xfce_rc_simple_open (SERVICES_CONFFILE, TRUE);
    if (! rc)
        return NULL;

    str = g_shell_unquote (xfce_rc_read_entry (rc, "SERVICES_BLACKLIST", ""), NULL);
    xfce_rc_close (rc);

    if (g_utf8_strlen (str, -1) == 0)
        return NULL;

    a = g_strsplit (str, " ", 0);
    g_free (str);
    str = NULL;
    tmp = a;
    for (str=*tmp;str;str=*(++tmp))
        list = g_list_append (list, str);

    g_free (a);

    return list;
}

GList *
services_blacklist_add (GList *list, const gchar *service, gboolean action)
{
    GList *ret = NULL;

    if (services_is_blacklisted (list, service))
        return list;

    ret = g_list_append (list, g_strdup (service));
    _services_blacklist_write (ret);
    if (action)
        _service_run (service, FALSE);
    return ret;
}

GList *
services_blacklist_del (GList *list, const gchar *service, gboolean action)
{
    GList *ret = NULL;

    if (! (ret = services_is_blacklisted (list, service)))
        return list;

    ret = g_list_remove_link (list, ret);
    _services_blacklist_write (ret);
    if (action)
        _service_run (service, TRUE);
    return ret;
}

void
services_blacklist_free (GList *list)
{
    g_list_foreach (list, _list_free, NULL);
    g_list_free (list);
}

